@@ -31,7 +31,6 @@ This uses the persistent tags to communicate from dm-script to python.
 Note that this is not thread-safe!
 """
 
-from .execdmscript import exec_dmscript
-from .execdmscript import get_dm_type
-from .execdmscript import get_python_type
-from .execdmscript import DMScriptWrapper
\ No newline at end of file
+from .execdmscript import *
+
+__version__ = "1.1.3"
\ No newline at end of file
